package com.sun.xml.internal.fastinfoset.sax;

import com.sun.xml.internal.fastinfoset.QualifiedName;
import com.sun.xml.internal.fastinfoset.util.LocalNameQualifiedNamesMap;
import com.sun.xml.internal.org.jvnet.fastinfoset.sax.EncodingAlgorithmAttributes;
import org.xml.sax.Locator;
import com.sun.xml.internal.fastinfoset.CommonResourceBundle;
import com.sun.xml.internal.org.jvnet.fastinfoset.FastInfosetException;
import org.xml.sax.Attributes;
import java.io.IOException;
import org.xml.sax.SAXException;
import com.sun.xml.internal.org.jvnet.fastinfoset.sax.FastInfosetWriter;
import com.sun.xml.internal.fastinfoset.Encoder;

public class SAXDocumentSerializer extends Encoder implements FastInfosetWriter
{
    protected boolean _elementHasNamespaces;
    protected boolean _charactersAsCDATA;
    
    protected SAXDocumentSerializer(boolean v) {
        super(v);
        this._elementHasNamespaces = false;
        this._charactersAsCDATA = false;
    }
    
    public SAXDocumentSerializer() {
        this._elementHasNamespaces = false;
        this._charactersAsCDATA = false;
    }
    
    public void reset() {
        super.reset();
        this._elementHasNamespaces = false;
        this._charactersAsCDATA = false;
    }
    
    public final void startDocument() throws SAXException {
        try {
            this.reset();
            this.encodeHeader(false);
            this.encodeInitialVocabulary();
        }
        catch (IOException e) {
            throw new SAXException("startDocument", e);
        }
    }
    
    public final void endDocument() throws SAXException {
        try {
            this.encodeDocumentTermination();
        }
        catch (IOException e) {
            throw new SAXException("endDocument", e);
        }
    }
    
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        try {
            if (!this._elementHasNamespaces) {
                this.encodeTermination();
                this.mark();
                this._elementHasNamespaces = true;
                this.write(56);
            }
            this.encodeNamespaceAttribute(prefix, uri);
        }
        catch (IOException e) {
            throw new SAXException("startElement", e);
        }
    }
    
    public final void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        int attributeCount = (atts != null && atts.getLength() > 0) ? this.countAttributes(atts) : 0;
        try {
            if (this._elementHasNamespaces) {
                this._elementHasNamespaces = false;
                if (attributeCount > 0) {
                    byte[] octetBuffer = this._octetBuffer;
                    int markIndex = this._markIndex;
                    octetBuffer[markIndex] |= 0x40;
                }
                this.resetMark();
                this.write(240);
                this._b = 0;
            }
            else {
                this.encodeTermination();
                this._b = 0;
                if (attributeCount > 0) {
                    this._b |= 0x40;
                }
            }
            this.encodeElement(namespaceURI, qName, localName);
            if (attributeCount > 0) {
                this.encodeAttributes(atts);
            }
        }
        catch (IOException e) {
            throw new SAXException("startElement", e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException("startElement", e2);
        }
    }
    
    public final void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        try {
            this.encodeElementTermination();
        }
        catch (IOException e) {
            throw new SAXException("endElement", e);
        }
    }
    
    public final void characters(char[] ch, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        if (this.getIgnoreWhiteSpaceTextContent() && isWhiteSpace(ch, start, length)) {
            return;
        }
        try {
            this.encodeTermination();
            if (!this._charactersAsCDATA) {
                this.encodeCharacters(ch, start, length);
            }
            else {
                this.encodeCIIBuiltInAlgorithmDataAsCDATA(ch, start, length);
            }
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public final void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        if (this.getIgnoreWhiteSpaceTextContent()) {
            return;
        }
        this.characters(ch, start, length);
        return;
    }
    
    public final void processingInstruction(String target, String data) throws SAXException {
        try {
            if (this.getIgnoreProcesingInstructions()) {
                return;
            }
            if (target.length() == 0) {
                throw new SAXException(CommonResourceBundle.getInstance().getString("message.processingInstructionTargetIsEmpty"));
            }
            this.encodeTermination();
            this.encodeProcessingInstruction(target, data);
        }
        catch (IOException e) {
            throw new SAXException("processingInstruction", e);
        }
    }
    
    public final void setDocumentLocator(Locator locator) {
    }
    
    public final void skippedEntity(String name) throws SAXException {
    }
    
    public final void comment(char[] ch, int start, int length) throws SAXException {
        try {
            if (this.getIgnoreComments()) {
                return;
            }
            this.encodeTermination();
            this.encodeComment(ch, start, length);
        }
        catch (IOException e) {
            throw new SAXException("startElement", e);
        }
    }
    
    public final void startCDATA() throws SAXException {
        this._charactersAsCDATA = true;
    }
    
    public final void endCDATA() throws SAXException {
        this._charactersAsCDATA = false;
    }
    
    public final void startDTD(String name, String publicId, String systemId) throws SAXException {
        if (this.getIgnoreDTD()) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeDocumentTypeDeclaration(publicId, systemId);
            this.encodeElementTermination();
        }
        catch (IOException e) {
            throw new SAXException("startDTD", e);
        }
        return;
    }
    
    public final void endDTD() throws SAXException {
    }
    
    public final void startEntity(String name) throws SAXException {
    }
    
    public final void endEntity(String name) throws SAXException {
    }
    
    public final void octets(String URI, int id, byte[] b, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeNonIdentifyingStringOnThirdBit(URI, id, b, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public final void object(String URI, int id, Object data) throws SAXException {
        try {
            this.encodeTermination();
            this.encodeNonIdentifyingStringOnThirdBit(URI, id, data);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
    }
    
    public final void bytes(byte[] b, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeCIIOctetAlgorithmData(1, b, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        return;
    }
    
    public final void shorts(short[] s, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeCIIBuiltInAlgorithmData(2, s, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public final void ints(int[] i, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeCIIBuiltInAlgorithmData(3, i, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public final void longs(long[] l, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeCIIBuiltInAlgorithmData(4, l, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public final void booleans(boolean[] b, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeCIIBuiltInAlgorithmData(5, b, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public final void floats(float[] f, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeCIIBuiltInAlgorithmData(6, f, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public final void doubles(double[] d, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeCIIBuiltInAlgorithmData(7, d, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public void uuids(long[] msblsb, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            this.encodeCIIBuiltInAlgorithmData(8, msblsb, start, length);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public void numericCharacters(char[] ch, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            boolean addToTable = this.isCharacterContentChunkLengthMatchesLimit(length);
            this.encodeNumericFourBitCharacters(ch, start, length, addToTable);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public void dateTimeCharacters(char[] ch, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            boolean addToTable = this.isCharacterContentChunkLengthMatchesLimit(length);
            this.encodeDateTimeFourBitCharacters(ch, start, length, addToTable);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public void alphabetCharacters(String alphabet, char[] ch, int start, int length) throws SAXException {
        if (length <= 0) {
            return;
        }
        try {
            this.encodeTermination();
            boolean addToTable = this.isCharacterContentChunkLengthMatchesLimit(length);
            this.encodeAlphabetCharacters(alphabet, ch, start, length, addToTable);
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    public void characters(char[] ch, int start, int length, boolean index) throws SAXException {
        if (length <= 0) {
            return;
        }
        if (this.getIgnoreWhiteSpaceTextContent() && isWhiteSpace(ch, start, length)) {
            return;
        }
        try {
            this.encodeTermination();
            if (!this._charactersAsCDATA) {
                this.encodeNonIdentifyingStringOnThirdBit(ch, start, length, this._v.characterContentChunk, index, true);
            }
            else {
                this.encodeCIIBuiltInAlgorithmDataAsCDATA(ch, start, length);
            }
        }
        catch (IOException e) {
            throw new SAXException(e);
        }
        catch (FastInfosetException e2) {
            throw new SAXException(e2);
        }
        return;
    }
    
    protected final int countAttributes(Attributes atts) {
        int count = 0;
        for (int i = 0; i < atts.getLength(); ++i) {
            String uri = atts.getURI(i);
            if (uri != "http://www.w3.org/2000/xmlns/" && !uri.equals("http://www.w3.org/2000/xmlns/")) {
                ++count;
            }
        }
        return count;
    }
    
    protected void encodeAttributes(Attributes atts) throws IOException, FastInfosetException {
        if ((atts instanceof EncodingAlgorithmAttributes)) {
            EncodingAlgorithmAttributes eAtts = (EncodingAlgorithmAttributes)atts;
            for (int j = 0; j < eAtts.getLength(); ++j) {
                if (this.encodeAttribute(atts.getURI(j), atts.getQName(j), atts.getLocalName(j))) {
                    Object data = eAtts.getAlgorithmData(j);
                    if (data == null) {
                        String value = eAtts.getValue(j);
                        boolean addToTable = this.isAttributeValueLengthMatchesLimit(value.length());
                        boolean mustBeAddedToTable = eAtts.getToIndex(j);
                        String alphabet = eAtts.getAlpababet(j);
                        if (alphabet == null) {
                            this.encodeNonIdentifyingStringOnFirstBit(value, this._v.attributeValue, addToTable, mustBeAddedToTable);
                        }
                        else if (alphabet == "0123456789-:TZ ") {
                            this.encodeDateTimeNonIdentifyingStringOnFirstBit(value, addToTable, mustBeAddedToTable);
                        }
                        else if (alphabet == "0123456789-+.E ") {
                            this.encodeNumericNonIdentifyingStringOnFirstBit(value, addToTable, mustBeAddedToTable);
                        }
                        else {
                            this.encodeNonIdentifyingStringOnFirstBit(value, this._v.attributeValue, addToTable, mustBeAddedToTable);
                        }
                    }
                    else {
                        this.encodeNonIdentifyingStringOnFirstBit(eAtts.getAlgorithmURI(j), eAtts.getAlgorithmIndex(j), data);
                    }
                }
            }
        }
        else {
            for (int i = 0; i < atts.getLength(); ++i) {
                if (this.encodeAttribute(atts.getURI(i), atts.getQName(i), atts.getLocalName(i))) {
                    String value = atts.getValue(i);
                    boolean addToTable = this.isAttributeValueLengthMatchesLimit(value.length());
                    this.encodeNonIdentifyingStringOnFirstBit(value, this._v.attributeValue, addToTable, false);
                }
            }
        }
        this._b = 240;
        this._terminate = true;
    }
    
    protected void encodeElement(String namespaceURI, String qName, String localName) throws IOException {
        LocalNameQualifiedNamesMap.Entry entry = this._v.elementName.obtainEntry(qName);
        if (entry._valueIndex > 0) {
            QualifiedName[] names = entry._value;
            int i = 0;
            while (i < entry._valueIndex) {
                QualifiedName n = names[i];
                if (namespaceURI == n.namespaceName || namespaceURI.equals(n.namespaceName)) {
                    this.encodeNonZeroIntegerOnThirdBit(names[i].index);
                    return;
                }
                ++i;
            }
        }
        this.encodeLiteralElementQualifiedNameOnThirdBit(namespaceURI, getPrefixFromQualifiedName(qName), localName, entry);
    }
    
    protected boolean encodeAttribute(String namespaceURI, String qName, String localName) throws IOException {
        LocalNameQualifiedNamesMap.Entry entry = this._v.attributeName.obtainEntry(qName);
        if (entry._valueIndex > 0) {
            QualifiedName[] names = entry._value;
            int i = 0;
            while (i < entry._valueIndex) {
                if (namespaceURI == names[i].namespaceName || namespaceURI.equals(names[i].namespaceName)) {
                    this.encodeNonZeroIntegerOnSecondBitFirstBitZero(names[i].index);
                    return true;
                }
                ++i;
            }
        }
        return this.encodeLiteralAttributeQualifiedNameOnSecondBit(namespaceURI, getPrefixFromQualifiedName(qName), localName, entry);
    }
}
